@@ -1,7 +1,9 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 2001, 2002
-// Copyright Dirk Lemstra 2013-2015
+//
+// Copyright @ 2013 ImageMagick Studio LLC, a non-profit organization
+// dedicated to making software imaging solutions freely available.
 //
 // CoderInfo implementation
 //
@@ -16,32 +18,41 @@
 using namespace std;
 
 Magick::CoderInfo::CoderInfo(void)
-  : _name(),
+  : _decoderThreadSupport(false),
     _description(),
-    _mimeType(),
+    _encoderThreadSupport(false),
+    _isMultiFrame(false),
     _isReadable(false),
     _isWritable(false),
-    _isMultiFrame(false)
+    _mimeType(),
+    _module(),
+    _name()
 {
 }
 
 Magick::CoderInfo::CoderInfo(const Magick::CoderInfo &coder_)
+  : _decoderThreadSupport(coder_._decoderThreadSupport),
+    _description(coder_._description),
+    _encoderThreadSupport(coder_._encoderThreadSupport),
+    _isMultiFrame(coder_._isMultiFrame),
+    _isReadable(coder_._isReadable),
+    _isWritable(coder_._isWritable),
+    _mimeType(coder_._mimeType),
+    _module(coder_._module),
+    _name(coder_._name)
 {
-  _name=coder_._name;
-  _description=coder_._description;
-  _mimeType=coder_._mimeType;
-  _isReadable=coder_._isReadable;
-  _isWritable=coder_._isWritable;
-  _isMultiFrame=coder_._isMultiFrame;
 }
 
 Magick::CoderInfo::CoderInfo(const std::string &name_)
-  : _name(),
+  : _decoderThreadSupport(false),
     _description(),
-    _mimeType(),
+    _encoderThreadSupport(false),
+    _isMultiFrame(false),
     _isReadable(false),
     _isWritable(false),
-    _isMultiFrame(false)
+    _mimeType(),
+    _module(),
+    _name()
 {
   const Magick::MagickInfo
     *magickInfo;
@@ -50,17 +61,24 @@ Magick::CoderInfo::CoderInfo(const std::string &name_)
   magickInfo=GetMagickInfo(name_.c_str(),exceptionInfo);
   ThrowPPException(false);
   if (magickInfo == 0)
-    {
-      throwExceptionExplicit(OptionError,"Coder not found",name_.c_str());
-    }
+    throwExceptionExplicit(MagickCore::OptionError,"Coder not found",
+      name_.c_str());
   else
     {
-      _name=std::string(magickInfo->name);
+      _decoderThreadSupport=(GetMagickDecoderThreadSupport(magickInfo) ==
+        MagickTrue) ? true : false;
       _description=std::string(magickInfo->description);
-      _mimeType=std::string(magickInfo->mime_type ? magickInfo->mime_type : "");
-      _isReadable=((magickInfo->decoder == 0) ? false : true);
-      _isWritable=((magickInfo->encoder == 0) ? false : true);
-      _isMultiFrame=((magickInfo->adjoin == 0) ? false : true);
+      _encoderThreadSupport=(GetMagickEncoderThreadSupport(magickInfo) ==
+        MagickTrue) ? true : false;
+      _isMultiFrame=(GetMagickAdjoin(magickInfo) == MagickTrue) ? true : false;
+      _isReadable=((magickInfo->decoder == (MagickCore::DecodeImageHandler *)
+        NULL) ? false : true);
+      _isWritable=((magickInfo->encoder == (MagickCore::EncodeImageHandler *)
+        NULL) ? false : true);
+      _mimeType=std::string(magickInfo->mime_type != (char *) NULL ?
+        magickInfo->mime_type : "");
+      _module=std::string(magickInfo->magick_module);
+      _name=std::string(magickInfo->name);
     }
 }
 
@@ -73,16 +91,29 @@ Magick::CoderInfo& Magick::CoderInfo::operator=(const CoderInfo &coder_)
   // If not being set to ourself
   if (this != &coder_)
     {
-      _name=coder_._name;
+      _decoderThreadSupport=coder_._decoderThreadSupport;
       _description=coder_._description;
-      _mimeType=coder_._mimeType;
+      _encoderThreadSupport=coder_._encoderThreadSupport;
+      _isMultiFrame=coder_._isMultiFrame;
       _isReadable=coder_._isReadable;
       _isWritable=coder_._isWritable;
-      _isMultiFrame=coder_._isMultiFrame;
+      _mimeType=coder_._mimeType;
+      _module=coder_._module;
+      _name=coder_._name;
     }
   return(*this);
 }
 
+bool Magick::CoderInfo::canReadMultithreaded(void) const
+{
+  return(_decoderThreadSupport);
+}
+
+bool Magick::CoderInfo::canWriteMultithreaded(void) const
+{
+  return(_encoderThreadSupport);
+}
+
 std::string Magick::CoderInfo::description(void) const
 {
   return(_description);
@@ -108,6 +139,11 @@ std::string Magick::CoderInfo::mimeType(void) const
   return(_mimeType);
 }
 
+std::string Magick::CoderInfo::module(void) const
+{
+  return(_module);
+}
+
 std::string Magick::CoderInfo::name(void) const
 {
   return(_name);
@@ -117,13 +153,3 @@ bool Magick::CoderInfo::unregister(void) const
 {
   return(UnregisterMagickInfo(_name.c_str()) != MagickFalse);
 }
-
-Magick::CoderInfo::CoderInfo(const MagickCore::MagickInfo *magickInfo_)
-  : _name(std::string(magickInfo_->name ? magickInfo_->name : "")),
-    _description(std::string(magickInfo_->description ? magickInfo_->description : "")),
-    _mimeType(std::string(magickInfo_->mime_type ? magickInfo_->mime_type : "")),
-    _isReadable(magickInfo_->decoder ? true : false),
-    _isWritable(magickInfo_->encoder ? true : false),
-    _isMultiFrame(magickInfo_->adjoin ? true : false)
-{
-}
