@@ -176,8 +176,6 @@ MagickExport CacheView *AcquireVirtualCacheView(const Image *image,
   cache_view->debug=(GetLogEventMask() & CacheEvent) != 0 ? MagickTrue :
     MagickFalse;
   cache_view->signature=MagickCoreSignature;
-  if (cache_view->nexus_info == (NexusInfo **) NULL)
-    ThrowFatalException(CacheFatalError,"UnableToAcquireCacheView");
   return(cache_view);
 }
 
