@@ -22,6 +22,11 @@
   MagickCoderHeader("AVIF", 4, "ftypavis") \
   MagickCoderHeader("HEIC", 4, "ftypheic") \
   MagickCoderHeader("HEIC", 4, "ftypheix") \
+  MagickCoderHeader("HEIC", 4, "ftyphevc") \
+  MagickCoderHeader("HEIC", 4, "ftypheim") \
+  MagickCoderHeader("HEIC", 4, "ftypheis") \
+  MagickCoderHeader("HEIC", 4, "ftyphevm") \
+  MagickCoderHeader("HEIC", 4, "ftyphevs") \
   MagickCoderHeader("HEIC", 4, "ftypmif1") \
   MagickCoderHeader("HEIC", 4, "ftypmsf1")
 
