@@ -246,7 +246,7 @@ static char *PNMString(Image *image,char *string,const size_t extent)
           return((char *) NULL);
         break;
       }
-    string[i]=c;
+    string[i]=(char) c;
     if (c == '\n' || c == '\r')
       break;
   }
@@ -409,7 +409,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           do
           {
             if ((size_t) (p-keyword) < (MagickPathExtent-1))
-              *p++=c;
+              *p++=(char) c;
             c=ReadBlobByte(image);
           } while (isalnum((int) ((unsigned char) c)));
           *p='\0';
@@ -421,7 +421,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           while (isalnum((int) ((unsigned char) c)) || (c == '_'))
           {
             if ((size_t) (p-value) < (MagickPathExtent-1))
-              *p++=c;
+              *p++=(char) c;
             c=ReadBlobByte(image);
           }
           *p='\0';
@@ -1784,6 +1784,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
     *value;
 
   MagickBooleanType
+    added_newline,
     status;
 
   MagickOffsetType
@@ -2007,13 +2008,15 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '1':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PBM image.
         */
         (void) SetImageType(image,BilevelType,exception);
+        extent=1;
         q=pixels;
+        added_newline=MagickFalse;
         for (y=0; y < (ssize_t) image->rows; y++)
         {
           const Quantum
@@ -2027,20 +2030,28 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
             break;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
+            if (q != pixels)
+              {
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                    added_newline=MagickTrue;
+                  }
+              }
             *q++=(unsigned char) (GetPixelLuma(image,p) >= ((double)
               QuantumRange/2.0) ? '0' : '1');
-            if ((q-pixels+2) >= (ssize_t) sizeof(pixels))
-              {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
-              }
-            *q++=' ';
             p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
+          if (added_newline == MagickFalse)
+            {
+              *q++='\n';
+              (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+              q=pixels;
+            }
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -2059,7 +2070,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '2':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PGM image.
@@ -2072,6 +2083,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           else
             (void) WriteBlobString(image,"4294967295\n");
         q=pixels;
+        added_newline=MagickFalse;
         for (y=0; y < (ssize_t) image->rows; y++)
         {
           const Quantum
@@ -2087,29 +2099,38 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           {
             index=ClampToQuantum(GetPixelLuma(image,p));
             if (image->depth <= 8)
-              count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,"%u ",
+              count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,"%u",
                 ScaleQuantumToChar(index));
             else
               if (image->depth <= 16)
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u ",ScaleQuantumToShort(index));
+                  "%u",ScaleQuantumToShort(index));
               else
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u ",ScaleQuantumToLong(index));
+                  "%u",ScaleQuantumToLong(index));
             extent=(size_t) count;
-            if ((size_t) (q-pixels+(ssize_t) extent+1) >= sizeof(pixels))
+            if (q != pixels)
               {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                    added_newline=MagickTrue;
+                  }
               }
             (void) memcpy((char *) q,buffer,extent);
             q+=(ptrdiff_t) extent;
             p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
+          if (added_newline == MagickFalse)
+            {
+              *q++='\n';
+              (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+              q=pixels;
+            }
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -2128,7 +2149,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '3':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PNM image.
@@ -2143,6 +2164,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           else
             (void) WriteBlobString(image,"4294967295\n");
         q=pixels;
+        added_newline=MagickFalse;
         for (y=0; y < (ssize_t) image->rows; y++)
         {
           const Quantum
@@ -2158,34 +2180,43 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           {
             if (image->depth <= 8)
               count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                "%u %u %u ",ScaleQuantumToChar(GetPixelRed(image,p)),
+                "%u %u %u",ScaleQuantumToChar(GetPixelRed(image,p)),
                 ScaleQuantumToChar(GetPixelGreen(image,p)),
                 ScaleQuantumToChar(GetPixelBlue(image,p)));
             else
               if (image->depth <= 16)
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u %u %u ",ScaleQuantumToShort(GetPixelRed(image,p)),
+                  "%u %u %u",ScaleQuantumToShort(GetPixelRed(image,p)),
                   ScaleQuantumToShort(GetPixelGreen(image,p)),
                   ScaleQuantumToShort(GetPixelBlue(image,p)));
               else
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u %u %u ",ScaleQuantumToLong(GetPixelRed(image,p)),
+                  "%u %u %u",ScaleQuantumToLong(GetPixelRed(image,p)),
                   ScaleQuantumToLong(GetPixelGreen(image,p)),
                   ScaleQuantumToLong(GetPixelBlue(image,p)));
             extent=(size_t) count;
-            if ((size_t) (q-pixels+(ssize_t) extent+2) >= sizeof(pixels))
+            if (q != pixels)
               {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                    added_newline=MagickTrue;
+                  }
               }
             (void) memcpy((char *) q,buffer,extent);
             q+=(ptrdiff_t) extent;
             p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
+          if (added_newline == MagickFalse)
+            {
+              *q++='\n';
+              (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+              q=pixels;
+            }
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
