@@ -61,6 +61,7 @@
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
 #include "MagickCore/module.h"
 #include "MagickCore/utility.h"
 
@@ -237,12 +238,12 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   height=0;
   *name='\0';
   while (ReadBlobString(image,buffer) != (char *) NULL)
-    if (sscanf(buffer,"#define %1024s %ld",name,&width) == 2)
+    if (MagickSscanf(buffer,"#define %1024s %ld",name,&width) == 2)
       if ((strlen(name) >= 6) &&
           (LocaleCompare(name+strlen(name)-6,"_width") == 0))
         break;
   while (ReadBlobString(image,buffer) != (char *) NULL)
-    if (sscanf(buffer,"#define %1024s %ld",name,&height) == 2)
+    if (MagickSscanf(buffer,"#define %1024s %ld",name,&height) == 2)
       if ((strlen(name) >= 7) &&
           (LocaleCompare(name+strlen(name)-7,"_height") == 0))
         break;
@@ -260,11 +261,11 @@ static Image *ReadXBMImage(const ImageInfo *image_info,ExceptionInfo *exception)
   offset=TellBlob(image);
   while (ReadBlobString(image,buffer) != (char *) NULL)
   {
-    if (sscanf(buffer,"static short %1024s = {",name) == 1)
+    if (MagickSscanf(buffer,"static short %1024s = {",name) == 1)
       version=10;
-    else if (sscanf(buffer,"static unsigned char %1024s = {",name) == 1)
+    else if (MagickSscanf(buffer,"static unsigned char %1024s = {",name) == 1)
       version=11;
-    else if (sscanf(buffer,"static char %1024s = {",name) == 1)
+    else if (MagickSscanf(buffer,"static char %1024s = {",name) == 1)
       version=11;
     else
       {
